@@ -94,7 +94,7 @@ const basic & basic::operator=(const basic & other)
 //////////
 
 /** Construct object from archive_node. */
-basic::basic(const archive_node &n, lst&) : flags(0)
+basic::basic(const archive_node &n, lst& /*unused*/) : flags(0)
 {
 	// Reconstruct tinfo_key from class name
 	std::string found_class_name;
@@ -149,14 +149,14 @@ next_context:
 
 		// Method not found, try parent print_context class
 		const print_context_class_info * parent_pc_info = pc_info->get_parent();
-		if (parent_pc_info) {
+		if (parent_pc_info != nullptr) {
 			pc_info = parent_pc_info;
 			goto next_context;
 		}
 
 		// Method still not found, try parent class
 		const registered_class_info * parent_reg_info = reg_info->get_parent();
-		if (parent_reg_info) {
+		if (parent_reg_info != nullptr) {
 			reg_info = parent_reg_info;
 			pc_info = &c.get_class_info();
 			goto next_class;
@@ -177,7 +177,7 @@ next_context:
 }
 
 /** Default output to stream. */
-void basic::do_print(const print_context & c, unsigned) const
+void basic::do_print(const print_context & c, unsigned /*unused*/) const
 {
 	c.s << "[" << class_name() << " object]";
 }
@@ -187,7 +187,7 @@ void basic::do_print_tree(const print_tree & c, unsigned level) const
 {
 	c.s << std::string(level, ' ') << class_name() << " @" << this
 	    << std::hex << ", hash=0x" << hashvalue << ", flags=0x" << flags << std::dec;
-	if (nops())
+	if (nops() != 0u)
 		c.s << ", nops=" << nops();
 	c.s << std::endl;
 	for (size_t i=0; i<nops(); ++i)
@@ -195,7 +195,7 @@ void basic::do_print_tree(const print_tree & c, unsigned level) const
 }
 
 /** Python parsable output to stream. */
-void basic::do_print_python_repr(const print_python_repr & c, unsigned) const
+void basic::do_print_python_repr(const print_python_repr & c, unsigned /*unused*/) const
 {
 	c.s << class_name() << "()";
 }
@@ -230,7 +230,7 @@ unsigned basic::precedence() const
 /** Information about the object.
  *
  *  @see class info_flags */
-bool basic::info(unsigned) const
+bool basic::info(unsigned /*unused*/) const
 {
 	// all possible properties are false for basic objects
 	return false;
@@ -246,13 +246,13 @@ size_t basic::nops() const
 }
 
 /** Return operand/member at position i. */
-ex basic::op(size_t) const
+const ex basic::op(size_t /*unused*/) const
 {
 	throw(std::range_error(std::string("basic::op(): ") + class_name() + std::string(" has no operands")));
 }
 
 /** Return modifyable operand/member at position i. */
-ex & basic::let_op(size_t)
+ex & basic::let_op(size_t /*unused*/)
 {
 	ensure_if_modifiable();
 	throw(std::range_error(std::string("basic::let_op(): ") + class_name() + std::string(" has no operands")));
@@ -319,7 +319,7 @@ ex basic::map(map_function & f) const
 		}
 	}
 
-	if (copy) {
+	if (copy != nullptr) {
 		copy->setflag(status_flags::dynallocated);
 		copy->clearflag(status_flags::hash_calculated | status_flags::expanded);
 		return *copy;
@@ -422,7 +422,7 @@ ex basic::collect(const ex & s, bool distributed) const
 }
 
 /** Perform automatic non-interruptive term rewriting rules. */
-ex basic::eval(int) const
+ex basic::eval(int /*unused*/) const
 {
 	// There is nothing to do for basic objects:
 	return hold();
@@ -453,18 +453,10 @@ ex basic::evalf(int level, PyObject* parent) const
 	}
 }
 
-/** Function object to be applied by basic::evalm(). */
-struct evalm_map_function : public map_function {
-	ex operator()(const ex & e) override { return evalm(e); }
-} map_evalm;
-
 /** Evaluate sums, products and integer powers of matrices. */
 ex basic::evalm() const
 {
-	if (nops() == 0)
-		return *this;
-	else
-		return map(map_evalm);
+	return *this;
 }
 
 /** Function object to be applied by basic::eval_integ(). */
@@ -528,7 +520,7 @@ ex basic::scalar_mul_indexed(const ex & self, const numeric & other) const
  *  @param v The complete vector of factors
  *  @return true if the contraction was successful, false otherwise
  *  @see ex::simplify_indexed() */
-bool basic::contract_with(exvector::iterator, exvector::iterator, exvector&) const
+bool basic::contract_with(exvector::iterator /*unused*/, exvector::iterator /*unused*/, exvector& /*unused*/) const
 {
 	// Do nothing
 	return false;
@@ -599,9 +591,14 @@ bool basic::match(const ex & pattern, lst & repl_lst) const
 /** Helper function for subs(). Does not recurse into subexpressions. */
 ex basic::subs_one_level(const exmap & m, unsigned options) const
 {
-	if (options & subs_options::no_pattern) {
-		const auto& it = m.find(*this);
-		return (it != m.end())? it->second : *this;
+	exmap::const_iterator it;
+
+        if (options & subs_options::no_pattern) {
+		ex thisex = *this;
+		it = m.find(thisex);
+		if (it != m.end())
+			return it->second;
+		return thisex;
 	} else {
                 for (const auto & elem : m) {
 			lst repl_lst;
@@ -618,7 +615,7 @@ ex basic::subs_one_level(const exmap & m, unsigned options) const
 ex basic::subs(const exmap & m, unsigned options) const
 {
 	size_t num = nops();
-	if (num) {
+	if (num != 0u) {
 
 		// Substitute in subexpressions
 		for (size_t i=0; i<num; i++) {
@@ -662,7 +659,7 @@ ex basic::diff(const symbol & s, unsigned nth) const
 		return ex(*this);
 	
 	// evaluate unevaluated *this before differentiating
-	if (!(flags & status_flags::evaluated))
+	if ((flags & status_flags::evaluated) == 0u)
 		return ex(*this).diff(s, nth);
 	
 	ex ndiff = this->derivative(s);
@@ -753,7 +750,7 @@ bool basic::is_equal_same_type(const basic & other) const
  *  automatically used instead of match_same_type() if nops() == 0.
  *
  *  @see basic::match */
-bool basic::match_same_type(const basic &) const
+bool basic::match_same_type(const basic & /*unused*/) const
 {
 	// The default is to only consider subexpressions, but not any other
 	// attributes
@@ -785,7 +782,7 @@ long basic::calchash() const
 	}
 
 	// store calculated hash value only if object is already evaluated
-	if (flags & status_flags::evaluated) {
+	if ((flags & status_flags::evaluated) != 0u) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
 	}
