@@ -34,7 +34,7 @@
 #include <vector>
 #include <list>
 #include <memory>
-#include <bits/algorithmfwd.h>
+#include <algorithm>
 
 namespace GiNaC {
 
@@ -130,7 +130,7 @@ template <template <class T, class = std::allocator<T> > class C>
 class container : public basic, public container_storage<C> {
 	GINAC_DECLARE_REGISTERED_CLASS(container, basic)
 
-	friend struct print_order;
+	friend class print_order;
 
 protected:
 	typedef typename container_storage<C>::STLT STLT;
