@@ -26,13 +26,13 @@
 #include "mul.h"
 #include "ncmul.h"
 #include "numeric.h"
-#include "matrix.h"
 #include "power.h"
 #include "lst.h"
 #include "function.h"
 #include "symbol.h"
 #include "relational.h"
 #include "utils.h"
+#include "operators.h"
 
 #include <iostream>
 #include <stdexcept>
@@ -75,7 +75,7 @@ void ex::dbgprinttree() const
 
 ex ex::expand(unsigned options) const
 {
-	if (options == 0 && (bp->flags & status_flags::expanded)) // The "expanded" flag only covers the standard options; someone might want to re-expand with different options
+	if (options == 0 && ((bp->flags & status_flags::expanded) != 0u)) // The "expanded" flag only covers the standard options; someone might want to re-expand with different options
 		return *this;
 	else
 		return bp->expand(options);
@@ -88,7 +88,7 @@ ex ex::expand(unsigned options) const
  *  @return partial derivative as a new expression */
 ex ex::diff(const symbol & s, unsigned nth) const
 {
-	if (!nth)
+	if (nth == 0u)
 		return *this;
 	else
 		return bp->diff(s, nth);
@@ -136,7 +136,7 @@ ex ex::subs(const lst & ls, const lst & lr, unsigned options) const
 		if (is_exactly_a<mul>(*its) || is_exactly_a<power>(*its))
 			options |= subs_options::pattern_is_product;
 	}
-	if (!(options & subs_options::pattern_is_product))
+	if ((options & subs_options::pattern_is_product) == 0u)
 		options |= subs_options::pattern_is_not_product;
 
 	return bp->subs(m, options);
@@ -179,7 +179,7 @@ ex ex::subs(const ex & e, unsigned options) const
 			if (is_exactly_a<mul>(s) || is_exactly_a<power>(s))
 				options |= subs_options::pattern_is_product;
 		}
-		if (!(options & subs_options::pattern_is_product))
+		if ((options & subs_options::pattern_is_product) == 0u)
 			options |= subs_options::pattern_is_not_product;
 
 		return bp->subs(m, options);
@@ -314,17 +314,6 @@ bool ex::is_zero() const {
         return ex_to<numeric>(*this).is_zero();
 }
 
-/** Check whether expression is zero or zero matrix. */
-bool ex::is_zero_matrix() const
-{
-	if (is_zero())
-		return  true;
-	else {
-		ex e = evalm();
-		return is_exactly_a<matrix>(e) && ex_to<matrix>(e).is_zero_matrix();
-	}
-}
-
 bool ex::is_integer_one() const
 {
     if (!is_exactly_a<numeric>(*this))
@@ -367,6 +356,45 @@ size_t ex::nsymbols() const
 	return res;
 }
 
+/** Return pointer to first symbol found in expression.  Due to GiNaC's
+ *  internal ordering of terms, it may not be obvious which symbol this
+ *  function returns for a given expression.
+ *
+ *  @param e  expression to search
+ *  @param x  first symbol found (returned)
+ *  @return "false" if no symbol was found, "true" otherwise */
+bool ex::get_first_symbol(ex &x) const
+{
+	if (is_exactly_a<symbol>(*this)) {
+		x = *this;
+		return true;
+	} else if (is_exactly_a<add>(*this) || is_exactly_a<mul>(*this)) {
+		for (size_t i=0; i<nops(); i++)
+			if (sorted_op(i).get_first_symbol(x))
+				return true;
+	} else if (is_exactly_a<power>(*this)) {
+		if (op(0).get_first_symbol(x))
+			return true;
+	}
+	return false;
+}
+
+static void collect_symbols(const ex& e, symbolset& syms)
+{
+	if (is_exactly_a<symbol>(e))
+		syms.insert(ex_to<symbol>(e));
+	else
+		for (size_t i=0; i < e.nops(); i++)
+                        collect_symbols(e.op(i), syms);
+}
+
+symbolset ex::symbols() const
+{
+        symbolset the_set;
+        collect_symbols(*this, the_set);
+	return the_set;
+}
+
 ex ex::sorted_op(size_t i) const
 {
 	if (is_a<expairseq>(*this))
@@ -374,6 +402,196 @@ ex ex::sorted_op(size_t i) const
 	else
 		return bp->op(i);
 }
+
+// Helper function: Return True if term is a monomial in symb.
+// If so, fill vec (applying map at the same time) with (coeff, expo) pairs.
+static bool match_monom(const ex& term, const symbol& symb,
+                expairvec& vec, const exmap& map)
+{
+        if (not has_free_symbol(term, symb)) {
+                return false;
+        }
+        if (term.is_equal(symb)) {
+                vec.push_back(std::make_pair(_ex1, _ex1));
+                return true;
+        }
+        if (is_exactly_a<power>(term)) {
+                const power& p = ex_to<power>(term);
+                const ex& expo = p.op(1);
+                if (p.op(0).is_equal(symb)) {
+                        vec.push_back(std::make_pair(_ex1, expo.subs(map)));
+                        return true;
+                }
+                else {
+                        // of form (...+...)^expo
+                        // we expand only those with integer exponent
+                        if (is_exactly_a<numeric>(expo)
+                                        and has_free_symbol(p.op(0), symb)) {
+                                numeric ee = ex_to<numeric>(expo);
+                                if (ee.is_integer() and ee.to_int() > 1) {
+                                        expairvec tmpvec;
+                                        expand(term).coefficients(symb, tmpvec);
+                                        for (const auto& pair : tmpvec)
+                                                vec.push_back(std::make_pair(pair.first.subs(map),
+                                                                        pair.second.subs(map)));
+                                        return true;
+                                }
+                        }
+                }
+        }
+        if (is_exactly_a<mul>(term)) {
+                // Handle cases C*x, C*x^c
+                for (const auto& mterm : term) {
+                        if (mterm.is_equal(symb)) {
+                                ex oth = ex_to<mul>(term).without_known_factor(symb);
+                                if (not has_free_symbol(oth, symb)) {
+                                        vec.push_back(std::make_pair(oth.subs(map), _ex1));
+                                        return true;
+                                }
+                        }
+                        if (is_exactly_a<power>(mterm)) {
+                                const power& p = ex_to<power>(mterm);
+                                if (p.op(0).is_equal(symb)) {
+                                        ex oth = ex_to<mul>(term).without_known_factor(mterm);
+                                        if (not has_free_symbol(oth, symb)) {
+                                                vec.push_back(std::make_pair(oth.subs(map), p.op(1).subs(map)));
+                                                return true;
+                                        }
+                                }
+                        }
+                }
+                // Handle C*(...+...)^c
+                for (const auto& mterm : term) {
+                        if (is_exactly_a<power>(mterm)
+                                        and is_exactly_a<numeric>(mterm.op(1))
+                                        and has_free_symbol(mterm.op(0), symb)) {
+                                numeric ee = ex_to<numeric>(mterm.op(1));
+                                if (ee.is_integer() and ee.to_int() > 1) {
+                                        expairvec tmpvec;
+                                        expand(term).coefficients(symb, tmpvec);
+                                        for (const auto& pair : tmpvec)
+                                                vec.push_back(std::make_pair(pair.first.subs(map),
+                                                        pair.second.subs(map)));
+                                        return true;
+                                }
+                        }
+                        if (is_exactly_a<add>(mterm)
+                                        and has_free_symbol(mterm, symb)) {
+                                expairvec tmpvec;
+                                expand(term).coefficients(symb, tmpvec);
+                                for (const auto& pair : tmpvec)
+                                        vec.push_back(std::make_pair(pair.first.subs(map),
+                                                pair.second.subs(map)));
+                                return true;
+                        }
+                }
+        }
+        return false;
+}
+
+/**
+ * Return in vec a list of pairs with (coefficient, exponent) of ex
+ * when interpreted as polynomial in s (with s any expression).
+ *
+ * We expand only parts of the expression known to contain free s, in
+ * order to keep coefficients as unexpanded as possible.
+ */
+void ex::coefficients(const ex & s, expairvec & vec) const
+{
+        vec.clear();
+        if (is_exactly_a<add>(s)) {
+                vec.push_back(std::make_pair(*this, _ex0));
+                return;
+        }
+
+        // Replace any s with a new anonymous symbol
+        symbol symb;
+        exmap submap {{s, symb}}, revmap {{symb, s}};
+        ex sub = subs(submap);
+
+        if (is_exactly_a<add>(sub)) {
+                const add& addref = ex_to<add>(sub);
+                const ex& oc = addref.op(addref.nops()+1);
+                if (not oc.is_zero())
+                        vec.push_back(std::make_pair(oc, _ex0));
+                for (const auto& term : addref.seq) {
+                        ex tmp = addref.recombine_pair_to_ex(term);
+                        if (not match_monom(tmp, symb, vec, revmap))
+                                vec.push_back(std::make_pair(tmp.subs(revmap), _ex0));
+                }
+        }
+        else {
+                if (not match_monom(sub, symb, vec, revmap)) {
+                        vec.clear();
+                        vec.push_back(std::make_pair(*this, _ex0));
+                }
+                return;
+        }
+
+        // Add: Combine coefficients with the same exponent
+        std::sort(vec.begin(), vec.end(), [](const std::pair<ex,ex>& x, const std::pair<ex,ex>& y)
+        {
+                return x.second < y.second;
+        });
+
+        auto tmp_it = vec.end();
+        for (auto it = vec.end(); it != vec.begin(); ) {
+                --it;
+                if ((it->first).is_zero()) {
+                        vec.erase(it);
+                        tmp_it = vec.end();
+                        continue;
+                }
+                if (tmp_it != vec.end() and (tmp_it->second).is_equal(it->second)) {
+                        it->first += tmp_it->first;
+                        vec.erase(tmp_it);
+                        if ((it->first).is_zero()) {
+                                vec.erase(it);
+                                tmp_it = vec.end();
+                                continue;
+                        }
+                }
+                tmp_it = it;
+        }
+
+}
+
+ex ex::deep_combine_fractions(ex e)
+{
+        if (is_a<expairseq>(e)) {
+                combine_map_function func;
+                e = e.map(func);
+        }
+        else {
+                if (is_exactly_a<symbol>(e) or
+                                is_exactly_a<constant>(e) or
+                                is_exactly_a<numeric>(e))
+                        return e;
+                else
+                        for (unsigned int i=0; i<e.nops(); ++i) {
+                                e.let_op(i) = deep_combine_fractions(e.op(i));
+                        }
+        }
+
+        if (is_exactly_a<add>(e)) {
+                ex t = ex_to<add>(e).combine_fractions();
+                return t;
+        }
+        else
+                return e;
+}
+
+ex ex::combine_fractions(bool deep) const
+{
+        if (not deep) {
+                if (is_exactly_a<add>(*this))
+                        return ex_to<add>(*this).combine_fractions();
+                else
+                        return *this;
+        }
+        return deep_combine_fractions(*this);
+}
+
 // private
 
 /** Make this ex writable (if more than one ex handle the same basic) by 
@@ -389,7 +607,7 @@ void ex::makewriteable()
  *  @see ex::compare(const ex &) */
 void ex::share(const ex & other) const
 {
-	if ((bp->flags | other.bp->flags) & status_flags::not_shareable)
+	if (((bp->flags | other.bp->flags) & status_flags::not_shareable) != 0u)
 		return;
 
 	if (bp->get_refcount() <= other.bp->get_refcount())
@@ -403,7 +621,7 @@ void ex::share(const ex & other) const
  *  @see ex::ex(const basic &) */
 ptr<basic> ex::construct_from_basic(const basic & other)
 {
-	if (!(other.flags & status_flags::evaluated)) {
+	if ((other.flags & status_flags::evaluated) == 0u) {
 
 		// The object is not yet evaluated, so call eval() to evaluate
 		// the top level. This will return either
@@ -430,7 +648,7 @@ ptr<basic> ex::construct_from_basic(const basic & other)
 		// it means that eval() hit case b) above. The original object is
 		// no longer needed (it evaluated into something different), so we
 		// delete it (because nobody else will).
-		if ((other.get_refcount() == 0) && (other.flags & status_flags::dynallocated))
+		if ((other.get_refcount() == 0) && ((other.flags & status_flags::dynallocated) != 0u))
 			delete &other; // yes, you can apply delete to a const pointer
 
 		// We can't return a basic& here because the tmpex is destroyed as
@@ -441,7 +659,7 @@ ptr<basic> ex::construct_from_basic(const basic & other)
 	} else {
 
 		// The easy case: making an "ex" out of an evaluated object.
-		if (other.flags & status_flags::dynallocated) {
+		if ((other.flags & status_flags::dynallocated) != 0u) {
 
 			// The object is already heap-allocated, so we can just make
 			// another reference to it.
