@@ -355,7 +355,7 @@ ex indexed::expand(unsigned options) const
 {
 	GINAC_ASSERT(seq.size() > 0);
 
-	if (options & expand_options::expand_indexed) {
+	if ((options & expand_options::expand_indexed) != 0u) {
 		ex newbase = seq[0].expand(options);
 		if (is_exactly_a<add>(newbase)) {
 			ex sum = _ex0;
@@ -408,7 +408,7 @@ void indexed::validate() const
 /** Implementation of ex::diff() for an indexed object always returns 0.
  *
  *  @see ex::diff */
-ex indexed::derivative(const symbol &) const
+ex indexed::derivative(const symbol & /*s*/) const
 {
 	return _ex0;
 }
@@ -538,7 +538,7 @@ struct is_summation_idx : public std::unary_function<ex, bool> {
 
 exvector integral::get_free_indices() const
 {
-	if (a.get_free_indices().size() || b.get_free_indices().size())
+	if ((a.get_free_indices().size() != 0u) || (b.get_free_indices().size() != 0u))
 		throw (std::runtime_error("integral::get_free_indices: boundary values should not have free indices"));
 	return f.get_free_indices();
 }
@@ -670,7 +670,7 @@ bool reposition_dummy_indices(ex & e, exvector & variant_dummy_indices, exvector
 		ex try_e = e;
 		for (size_t j=0; j<local_var_dummies.size(); ++j) {
 			exmap m;
-			if (1<<j & i) {
+			if ((1<<j & i) != 0u) {
 				ex curr_idx = local_var_dummies[j];
 				ex curr_toggle = ex_to<varidx>(curr_idx).toggle_variance();
 				m[curr_idx] = curr_toggle;
@@ -1276,7 +1276,7 @@ ex simplify_indexed(const ex & e, exvector & free_indices, exvector & dummy_indi
  *
  *  @param options Simplification options (currently unused)
  *  @return simplified expression */
-ex ex::simplify_indexed(unsigned) const
+ex ex::simplify_indexed(unsigned /*unused*/) const
 {
 	exvector free_indices, dummy_indices;
 	scalar_products sp;
@@ -1291,7 +1291,7 @@ ex ex::simplify_indexed(unsigned) const
  *  @param sp Scalar products to be replaced automatically
  *  @param options Simplification options (currently unused)
  *  @return simplified expression */
-ex ex::simplify_indexed(const scalar_products & sp, unsigned) const
+ex ex::simplify_indexed(const scalar_products & sp, unsigned /*unused*/) const
 {
 	exvector free_indices, dummy_indices;
 	return GiNaC::simplify_indexed(*this, free_indices, dummy_indices, sp);
@@ -1350,10 +1350,10 @@ bool spmapkey::operator==(const spmapkey &other) const
 bool spmapkey::operator<(const spmapkey &other) const
 {
 	int cmp = v1.compare(other.v1);
-	if (cmp)
+	if (cmp != 0)
 		return cmp < 0;
 	cmp = v2.compare(other.v2);
-	if (cmp)
+	if (cmp != 0)
 		return cmp < 0;
 
 	// Objects are equal, now check dimensions
@@ -1378,7 +1378,7 @@ void scalar_products::add(const ex & v1, const ex & v2, const ex & dim, const ex
 	spm[spmapkey(v1, v2, dim)] = sp;
 }
 
-void scalar_products::add_vectors(const lst & l, const ex &)
+void scalar_products::add_vectors(const lst & l, const ex & /*unused*/)
 {
 	// Add all possible pairs of products
         for (const auto & elem1 : l)
