@@ -24,7 +24,6 @@
 #include "ex.h"
 #include "constant.h"
 #include "lst.h"
-#include "matrix.h"
 #include "mul.h"
 #include "power.h"
 #include "operators.h"
@@ -362,9 +361,9 @@ static ex step_series(const ex & arg,
                       unsigned options)
 {
 	const ex arg_pt = arg.subs(rel, subs_options::no_pattern);
-	if (arg_pt.info(info_flags::numeric)
+	if (is_exactly_a<numeric>(arg_pt)
 	    && ex_to<numeric>(arg_pt).real().is_zero()
-	    && !(options & series_options::suppress_branchcut))
+	    && ((options & series_options::suppress_branchcut) == 0u))
 		throw (std::domain_error("step_series(): on imaginary axis"));
 	
 	epvector seq;
@@ -443,7 +442,7 @@ static ex csgn_series(const ex & arg,
 	const ex arg_pt = arg.subs(rel, subs_options::no_pattern);
 	if (arg_pt.info(info_flags::numeric)
 	    && ex_to<numeric>(arg_pt).real().is_zero()
-	    && !(options & series_options::suppress_branchcut))
+	    && ((options & series_options::suppress_branchcut) == 0u))
 		throw (std::domain_error("csgn_series(): on imaginary axis"));
 	
 	epvector seq;
@@ -691,7 +690,7 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			return ser.series(rel, order);
 		}
 		// third special case: x real, >=1 (branch cut)
-		if (!(options & series_options::suppress_branchcut) &&
+		if (((options & series_options::suppress_branchcut) == 0u) &&
 			ex_to<numeric>(x_pt).is_real() && ex_to<numeric>(x_pt)>1) {
 			// method:
 			// This is the branch cut: assemble the primitive series manually
@@ -752,37 +751,6 @@ static ex Li3_eval(const ex & x)
 REGISTER_FUNCTION(Li3, eval_func(Li3_eval).
                        latex_name("{\\rm Li}_3"));
 
-//////////
-// Derivatives of Riemann's Zeta-function  zetaderiv(0,x)==zeta(x)
-//////////
-
-static ex zetaderiv_eval(const ex & n, const ex & x)
-{
-	if (n.info(info_flags::numeric)) {
-		// zetaderiv(0,x) -> zeta(x)
-		if (n.is_zero())
-			return zeta(x).hold();
-	}
-	
-	return zetaderiv(n, x).hold();
-}
-
-static ex zetaderiv_deriv(const ex & n, const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param<2);
-	
-	if (deriv_param==0) {
-		// d/dn zeta(n,x)
-		throw(std::logic_error("cannot diff zetaderiv(n,x) with respect to n"));
-	}
-	// d/dx psi(n,x)
-	return zetaderiv(n+1,x);
-}
-
-REGISTER_FUNCTION(zetaderiv, eval_func(zetaderiv_eval).
-	                       	 derivative_func(zetaderiv_deriv).
-  	                         latex_name("\\zeta^\\prime"));
-
 //////////
 // factorial
 //////////
@@ -986,195 +954,4 @@ REGISTER_FUNCTION(Order, eval_func(Order_eval).
                          real_part_func(Order_real_part).
                          imag_part_func(Order_imag_part));
 
-//////////
-// Solve linear system
-//////////
-
-ex lsolve(const ex &eqns, const ex &symbols, unsigned options)
-{
-	// solve a system of linear equations
-	if (eqns.info(info_flags::relation_equal)) {
-		if (!symbols.info(info_flags::symbol))
-			throw(std::invalid_argument("lsolve(): 2nd argument must be a symbol"));
-		const ex sol = lsolve(lst(eqns),lst(symbols));
-		
-		GINAC_ASSERT(sol.nops()==1);
-		GINAC_ASSERT(is_exactly_a<relational>(sol.op(0)));
-		
-		return sol.op(0).op(1); // return rhs of first solution
-	}
-	
-	// syntax checks
-	if (!eqns.info(info_flags::list)) {
-		throw(std::invalid_argument("lsolve(): 1st argument must be a list or an equation"));
-	}
-	for (size_t i=0; i<eqns.nops(); i++) {
-		if (!eqns.op(i).info(info_flags::relation_equal)) {
-			throw(std::invalid_argument("lsolve(): 1st argument must be a list of equations"));
-		}
-	}
-	if (!symbols.info(info_flags::list)) {
-		throw(std::invalid_argument("lsolve(): 2nd argument must be a list or a symbol"));
-	}
-	for (size_t i=0; i<symbols.nops(); i++) {
-		if (!symbols.op(i).info(info_flags::symbol)) {
-			throw(std::invalid_argument("lsolve(): 2nd argument must be a list of symbols"));
-		}
-	}
-	
-	// build matrix from equation system
-	matrix sys(eqns.nops(),symbols.nops());
-	matrix rhs(eqns.nops(),1);
-	matrix vars(symbols.nops(),1);
-	
-	for (size_t r=0; r<eqns.nops(); r++) {
-		const ex eq = eqns.op(r).op(0)-eqns.op(r).op(1); // lhs-rhs==0
-		ex linpart = eq;
-		for (size_t c=0; c<symbols.nops(); c++) {
-			const ex co = eq.coeff(ex_to<symbol>(symbols.op(c)),1);
-			linpart -= co*symbols.op(c);
-			sys(r,c) = co;
-		}
-		linpart = linpart.expand();
-		rhs(r,0) = -linpart;
-	}
-	
-	// test if system is linear and fill vars matrix
-	for (size_t i=0; i<symbols.nops(); i++) {
-		vars(i,0) = symbols.op(i);
-		if (sys.has(symbols.op(i)))
-			throw(std::logic_error("lsolve: system is not linear"));
-		if (rhs.has(symbols.op(i)))
-			throw(std::logic_error("lsolve: system is not linear"));
-	}
-	
-	matrix solution;
-	try {
-		solution = sys.solve(vars,rhs,options);
-	} catch (const std::runtime_error & e) {
-		// Probably singular matrix or otherwise overdetermined system:
-		// It is consistent to return an empty list
-		return lst();
-	}
-	GINAC_ASSERT(solution.cols()==1);
-	GINAC_ASSERT(solution.rows()==symbols.nops());
-	
-	// return list of equations of the form lst(var1==sol1,var2==sol2,...)
-	lst sollist;
-	for (size_t i=0; i<symbols.nops(); i++)
-		sollist.append(symbols.op(i)==solution(i,0));
-	
-	return sollist;
-}
-
-//////////
-// Find real root of f(x) numerically
-//////////
-
-const numeric
-fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2,
-		PyObject* parent)
-{
-	if (!x1.is_real() || !x2.is_real()) {
-		throw std::runtime_error("fsolve(): interval not bounded by real numbers");
-	}
-	if (x1==x2) {
-		throw std::runtime_error("fsolve(): vanishing interval");
-	}
-	// xx[0] == left interval limit, xx[1] == right interval limit.
-	// fx[0] == f(xx[0]), fx[1] == f(xx[1]).
-	// We keep the root bracketed: xx[0]<xx[1] and fx[0]*fx[1]<0.
-	numeric xx[2] = { x1<x2 ? x1 : x2,
-	                  x1<x2 ? x2 : x1 };
-	ex f;
-	if (is_a<relational>(f_in)) {
-		f = f_in.lhs()-f_in.rhs();
-	} else {
-		f = f_in;
-	}
-	const ex fx_[2] = { f.subs(x==xx[0]).evalf(0, parent),
-	                    f.subs(x==xx[1]).evalf(0, parent) };
-	if (!is_a<numeric>(fx_[0]) || !is_a<numeric>(fx_[1])) {
-		throw std::runtime_error("fsolve(): function does not evaluate numerically");
-	}
-	numeric fx[2] = { ex_to<numeric>(fx_[0]),
-	                  ex_to<numeric>(fx_[1]) };
-	if (!fx[0].is_real() || !fx[1].is_real()) {
-		throw std::runtime_error("fsolve(): function evaluates to complex values at interval boundaries");
-	}
-	if (fx[0]*fx[1]>=0) {
-		throw std::runtime_error("fsolve(): function does not change sign at interval boundaries");
-	}
-
-	// The Newton-Raphson method has quadratic convergence!  Simply put, it
-	// replaces x with x-f(x)/f'(x) at each step.  -f/f' is the delta:
-	const ex ff = normal(-f/f.diff(x));
-	int side = 0;  // Start at left interval limit.
-	numeric xxprev;
-	numeric fxprev;
-	do {
-		xxprev = xx[side];
-		fxprev = fx[side];
-		xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf(0,
-					parent));
-		fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf(0, parent));
-		if ((side==0 && xx[0]<xxprev) || (side==1 && xx[1]>xxprev) || xx[0]>xx[1]) {
-			// Oops, Newton-Raphson method shot out of the interval.
-			// Restore, and try again with the other side instead!
-			xx[side] = xxprev;
-			fx[side] = fxprev;
-			side = !side;
-			xxprev = xx[side];
-			fxprev = fx[side];
-			xx[side] += ex_to<numeric>(ff.subs(x==xx[side]).evalf(0,
-						parent));
-			fx[side] = ex_to<numeric>(f.subs(x==xx[side]).evalf(0,
-						parent));
-		}
-		if ((fx[side]<0 && fx[!side]<0) || (fx[side]>0 && fx[!side]>0)) {
-			// Oops, the root isn't bracketed any more.
-			// Restore, and perform a bisection!
-			xx[side] = xxprev;
-			fx[side] = fxprev;
-
-			// Ah, the bisection! Bisections converge linearly. Unfortunately,
-			// they occur pretty often when Newton-Raphson arrives at an x too
-			// close to the result on one side of the interval and
-			// f(x-f(x)/f'(x)) turns out to have the same sign as f(x) due to
-			// precision errors! Recall that this function does not have a
-			// precision goal as one of its arguments but instead relies on
-			// x converging to a fixed point. We speed up the (safe but slow)
-			// bisection method by mixing in a dash of the (unsafer but faster)
-			// secant method: Instead of splitting the interval at the
-			// arithmetic mean (bisection), we split it nearer to the root as
-			// determined by the secant between the values xx[0] and xx[1].
-			// Don't set the secant_weight to one because that could disturb
-			// the convergence in some corner cases!
-			static const double secant_weight = 0.984375;  // == 63/64 < 1
-			numeric xxmid = (1-secant_weight)*0.5*(xx[0]+xx[1])
-			    + secant_weight*(xx[0]+fx[0]*(xx[0]-xx[1])/(fx[1]-fx[0]));
-			numeric fxmid = ex_to<numeric>(f.subs(x==xxmid).evalf(0,
-						parent));
-			if (fxmid.is_zero()) {
-				// Luck strikes...
-				return xxmid;
-			}
-			if ((fxmid<0 && fx[side]>0) || (fxmid>0 && fx[side]<0)) {
-				side = !side;
-			}
-			xxprev = xx[side];
-			fxprev = fx[side];
-			xx[side] = xxmid;
-			fx[side] = fxmid;
-		}
-	} while (xxprev!=xx[side]);
-	return xxprev;
-}
-
-
-/* Force inclusion of functions from inifcns_gamma and inifcns_zeta
- * for static lib (so ginsh will see them). */
-unsigned force_include_tgamma = tgamma_SERIAL::serial;
-unsigned force_include_zeta1 = zeta1_SERIAL::serial;
-
 } // namespace GiNaC
