@@ -22,15 +22,18 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "lst.h"
 #include "constant.h"
 #include "infinity.h"
 #include "numeric.h"
+#include "mul.h"
 #include "power.h"
 #include "operators.h"
 #include "relational.h"
 #include "symbol.h"
 #include "pseries.h"
 #include "utils.h"
+#include "add.h"
 
 #include <vector>
 #include <stdexcept>
@@ -72,10 +75,11 @@ static ex exp_eval(const ex & x)
 	// exp(oo) -> oo
 	// exp(-oo) -> 0
 	// exp(UnsignedInfinity) -> error
-	if (x.info(info_flags::infinity)) {
-		if (x.is_equal(Infinity))
+	if (is_exactly_a<infinity>(x)) {
+	        infinity xinf = ex_to<infinity>(x);
+		if (xinf.is_plus_infinity())
 			return Infinity;
-		if (x.is_equal(NegInfinity))
+		if (xinf.is_minus_infinity())
 			return _ex0;
 		// x is UnsignedInfinity
 		throw (std::runtime_error("exp_eval(): exp^(unsigned_infinity) encountered"));
@@ -98,27 +102,44 @@ static ex exp_eval(const ex & x)
                 return false;
         };
 
+	ex x_red;
         if (has_pi_and_py(x)) { // true if x contains Pi and Python objects
                                 // like I. To make this check should be faster
                                 // than the following
-                ex res1 = sin(x/I);
-                ex res2 = cos(x/I);
-                if (((not is_exactly_a<function>(res1))
-                or (not is_ex_the_function(res1, sin)))
-                and ((not is_exactly_a<function>(res2))
-                or (not is_ex_the_function(res2, cos))))
-                        return I*res1 + res2;
+
+		ex coef_pi = x.coeff(Pi).expand();
+		ex rem = _ex0;
+		if (is_exactly_a<add>(coef_pi)) {
+			for (size_t i=0; i < coef_pi.nops(); i++) {
+				if ((coef_pi.op(i) / (_ex2 * I)).info(info_flags::integer))
+					rem += Pi * coef_pi.op(i);
+			}
+		}
+		else if ((coef_pi / (_ex2 * I)).info(info_flags::integer))
+			rem = Pi * coef_pi;
+		x_red = (x - rem).expand();
+
+
+		ex res1 = sin(x_red/I);
+		ex res2 = cos(x_red/I);
+		if (((not is_exactly_a<function>(res1))
+		or (not is_ex_the_function(res1, sin)))
+		and ((not is_exactly_a<function>(res2))
+		or (not is_ex_the_function(res2, cos))))
+			return I*res1 + res2;
         }
+	else
+		x_red = x;
 
 	// exp(log(x)) -> x
-	if (is_ex_the_function(x, log))
-		return x.op(0);
+	if (is_ex_the_function(x_red, log))
+		return x_red.op(0);
 	
 	// exp(float) -> float
-	if (x.info(info_flags::numeric) && !x.info(info_flags::crational))
-		return exp(ex_to<numeric>(x));
+	if (is_exactly_a<numeric>(x_red) && !x_red.info(info_flags::crational))
+		return exp(ex_to<numeric>(x_red));
 	
-	return exp(x).hold();
+	return exp(x_red).hold();
 }
 
 static ex exp_deriv(const ex & x, unsigned deriv_param)
@@ -227,7 +248,7 @@ static ex log_evalf(const ex & x, PyObject* parent)
 
 static ex log_eval(const ex & x)
 {
-	if (x.info(info_flags::numeric)) {
+	if (is_exactly_a<numeric>(x)) {
 		// log(float) -> float
 		if (!x.info(info_flags::crational))
 			return log(ex_to<numeric>(x));
@@ -334,7 +355,7 @@ static ex log_series(const ex &arg,
 				for (int i = order-1; i>0; --i) {
 					epvector cterm;
 					cterm.reserve(1);
-					cterm.push_back(expair(i%2 ? _ex1/i : _ex_1/i, _ex0));
+					cterm.push_back(expair((i%2) != 0 ? _ex1/i : _ex_1/i, _ex0));
 					acc = pseries(rel, cterm).add_series(ex_to<pseries>(acc));
 					acc = (ex_to<pseries>(rest)).mul_series(ex_to<pseries>(acc));
 				}
@@ -345,7 +366,7 @@ static ex log_series(const ex &arg,
 		} else  // it was a monomial
 			return pseries(rel, seq);
 	}
-	if (!(options & series_options::suppress_branchcut) &&
+	if (((options & series_options::suppress_branchcut) == 0u) &&
 	     arg_pt.info(info_flags::negative)) {
 		// method:
 		// This is the branch cut: assemble the primitive series manually and
@@ -372,7 +393,7 @@ static ex log_real_part(const ex & x)
 static ex log_imag_part(const ex & x)
 {
 	if (x.info(info_flags::positive))
-		return 0;
+		return _ex0;
 	return atan2(GiNaC::imag_part(x), GiNaC::real_part(x));
 }
 
@@ -400,574 +421,188 @@ REGISTER_FUNCTION(log, eval_func(log_eval).
                        latex_name("\\log"));
 
 //////////
-// hyperbolic sine (trigonometric function)
+// General logarithm
+// This only has shortcuts and delegates everything else to log(arg)
 //////////
 
-static ex sinh_evalf(const ex & x, PyObject* parent)
+static ex logb_evalf(const ex & x, const ex & base, PyObject* parent)
 {
-	if (is_exactly_a<numeric>(x))
-		return sinh(ex_to<numeric>(x));
-	
-	return sinh(x).hold();
+        if ((base - exp(_ex1).hold()).is_zero())
+                return log_evalf(x, parent);
+	if (is_exactly_a<numeric>(x) and is_exactly_a<numeric>(base))
+		return log(ex_to<numeric>(x), ex_to<numeric>(base));
+
+	return mul(log(x), pow(log(base), _ex_1));
 }
 
-static ex sinh_eval(const ex & x)
+static ex logb_eval(const ex & x, const ex & base)
 {
-	if (x.info(info_flags::numeric)) {
+	if (is_exactly_a<numeric>(x) and x.info(info_flags::crational)
+	    and is_exactly_a<numeric>(base) and base.info(info_flags::crational)) {
+                numeric a = ex_to<numeric>(x);
+                numeric b = ex_to<numeric>(base);
+                if (b.info(info_flags::real) and a.info(info_flags::real)) {
+                        bool israt;
+                        numeric ret = a.ratlog(b, israt);
+                        if (israt)
+                                return ret;
+                }
+                return mul(log(x), pow(log(base), _ex_1));
+        }
 
-		// sinh(0) -> 0
-		if (x.is_zero())
-			return _ex0;        
+        if ((base - exp(_ex1).hold()).is_zero())
+                return log_eval(x);
 
-		// sinh(float) -> float
-		if (!x.info(info_flags::crational))
-			return sinh(ex_to<numeric>(x));
-
-		// sinh() is odd
-		if (x.info(info_flags::negative))
-			return -sinh(-x);
+	// log(base^t, base) -> t
+	if (is_exactly_a<power>(x)) {
+                if (x.op(0).is_equal(base) and x.op(1).info(info_flags::real))
+			return x.op(1);
 	}
 
-	// sinh(oo) -> oo
-	// sinh(-oo) -> -oo
-	// sinh(UnsignedInfinity) -> error
+	// log(oo) -> oo
 	if (x.info(info_flags::infinity)) {
-		if (x.is_equal(UnsignedInfinity))
-			throw (std::runtime_error("sinh_eval(): sinh(unsigned_infinity) encountered"));
-		return x;
-	}
-	
-	if ((x/Pi).info(info_flags::numeric) &&
-		ex_to<numeric>(x/Pi).real().is_zero())  // sinh(I*x) -> I*sin(x)
-		return I*sin(x/I);
-	
-	if (is_exactly_a<function>(x)) {
-		const ex &t = x.op(0);
-
-		// sinh(asinh(x)) -> x
-		if (is_ex_the_function(x, asinh))
-			return t;
-
-		// sinh(acosh(x)) -> sqrt(x-1) * sqrt(x+1)
-		if (is_ex_the_function(x, acosh))
-			return sqrt(t-_ex1)*sqrt(t+_ex1);
-
-		// sinh(atanh(x)) -> x/sqrt(1-x^2)
-		if (is_ex_the_function(x, atanh))
-			return t*power(_ex1-power(t,_ex2),_ex_1_2);
-	}
-	
-	return sinh(x).hold();
-}
-
-static ex sinh_deriv(const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param==0);
-	
-	// d/dx sinh(x) -> cosh(x)
-	return cosh(x);
-}
-
-static ex sinh_real_part(const ex & x)
-{
-	return sinh(GiNaC::real_part(x))*cos(GiNaC::imag_part(x));
-}
-
-static ex sinh_imag_part(const ex & x)
-{
-	return cosh(GiNaC::real_part(x))*sin(GiNaC::imag_part(x));
-}
-
-static ex sinh_conjugate(const ex & x)
-{
-	// conjugate(sinh(x))==sinh(conjugate(x))
-	return sinh(x.conjugate());
-}
-
-REGISTER_FUNCTION(sinh, eval_func(sinh_eval).
-                        evalf_func(sinh_evalf).
-                        derivative_func(sinh_deriv).
-                        real_part_func(sinh_real_part).
-                        imag_part_func(sinh_imag_part).
-			conjugate_func(sinh_conjugate).
-		        latex_name("\\sinh"));
-
-//////////
-// hyperbolic cosine (trigonometric function)
-//////////
-
-static ex cosh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return cosh(ex_to<numeric>(x));
-	
-	return cosh(x).hold();
-}
-
-static ex cosh_eval(const ex & x)
-{
-	if (x.info(info_flags::numeric)) {
-
-		// cosh(0) -> 1
-		if (x.is_zero())
-			return _ex1;
-
-		// cosh(float) -> float
-		if (!x.info(info_flags::crational))
-			return cosh(ex_to<numeric>(x));
-
-		// cosh() is even
-		if (x.info(info_flags::negative))
-			return cosh(-x);
-	}
-	
-	// cosh(oo) -> oo
-	// cosh(-oo) -> oo
-	// cosh(UnsignedInfinity) -> error
-	if (x.info(info_flags::infinity)) {
-		if (x.is_equal(UnsignedInfinity))
-			throw (std::runtime_error("cosh_eval(): cosh(unsigned_infinity) encountered"));
 		return Infinity;
 	}
 
-	if ((x/Pi).info(info_flags::numeric) &&
-		ex_to<numeric>(x/Pi).real().is_zero())  // cosh(I*x) -> cos(x)
-		return cos(x/I);
-	
-	if (is_exactly_a<function>(x)) {
-		const ex &t = x.op(0);
+        // log(x)/log(base)
+	return mul(log(x), pow(log(base), _ex_1));
+}
 
-		// cosh(acosh(x)) -> x
-		if (is_ex_the_function(x, acosh))
-			return t;
+REGISTER_FUNCTION(logb, eval_func(logb_eval).
+                       evalf_func(logb_evalf).
+                       latex_name("\\log"));
 
-		// cosh(asinh(x)) -> sqrt(1+x^2)
-		if (is_ex_the_function(x, asinh))
-			return sqrt(_ex1+power(t,_ex2));
+//////////////////////////////////////
+// classical polylog function (see inifcns_nstdsum.cpp for multiple)
+//////////////////////////////////////
 
-		// cosh(atanh(x)) -> 1/sqrt(1-x^2)
-		if (is_ex_the_function(x, atanh))
-			return power(_ex1-power(t,_ex2),_ex_1_2);
+static ex Li_evalf(const ex& m_, const ex& x_, PyObject* parent)
+{
+	if (is_exactly_a<numeric>(m_) and is_exactly_a<numeric>(x_))
+                return Li2(ex_to<numeric>(m_), ex_to<numeric>(x_), parent);
+
+        return Li(m_,x_).hold();
+}
+
+
+static ex Li_eval(const ex& m_, const ex& x_)
+{
+	// classical polylogs
+	if (x_.is_zero()) {
+		return _ex0;
 	}
-	
-	return cosh(x).hold();
-}
-
-static ex cosh_deriv(const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param==0);
-	
-	// d/dx cosh(x) -> sinh(x)
-	return sinh(x);
-}
-
-static ex cosh_real_part(const ex & x)
-{
-	return cosh(GiNaC::real_part(x))*cos(GiNaC::imag_part(x));
-}
-
-static ex cosh_imag_part(const ex & x)
-{
-	return sinh(GiNaC::real_part(x))*sin(GiNaC::imag_part(x));
-}
-
-static ex cosh_conjugate(const ex & x)
-{
-	// conjugate(cosh(x))==cosh(conjugate(x))
-	return cosh(x.conjugate());
-}
-
-REGISTER_FUNCTION(cosh, eval_func(cosh_eval).
-                        evalf_func(cosh_evalf).
-                        derivative_func(cosh_deriv).
-                        real_part_func(cosh_real_part).
-                        imag_part_func(cosh_imag_part).
-                        conjugate_func(cosh_conjugate).
-                        latex_name("\\cosh"));
-
-//////////
-// hyperbolic tangent (trigonometric function)
-//////////
-
-static ex tanh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return tanh(ex_to<numeric>(x));
-	
-	return tanh(x).hold();
-}
-
-static ex tanh_eval(const ex & x)
-{
-	if (x.info(info_flags::numeric)) {
-
-		// tanh(0) -> 0
-		if (x.is_zero())
-			return _ex0;
-
-		// tanh(float) -> float
-		if (!x.info(info_flags::crational))
-			return tanh(ex_to<numeric>(x));
-
-		// tanh() is odd
-		if (x.info(info_flags::negative))
-			return -tanh(-x);
+	if (x_.is_integer_one()) {
+		return zeta(m_);
 	}
-	
-	// tanh(oo) -> 1
-	// tanh(-oo) -> -1
-	// tanh(UnsignedInfinity) -> error
-	if (x.info(info_flags::infinity)) {
-		if (x.is_equal(Infinity))
-			return _ex1;
-		if (x.is_equal(NegInfinity))
-			return _ex_1;
-		// x is UnsignedInfinity
-		throw (std::runtime_error("tanh_eval(): tanh(unsigned_infinity) encountered"));
+	if ((-x_).is_integer_one()) {
+		return (pow(2,1-m_)-1) * zeta(m_);
 	}
-		
-	if ((x/Pi).info(info_flags::numeric) &&
-		ex_to<numeric>(x/Pi).real().is_zero())  // tanh(I*x) -> I*tan(x);
-		return I*tan(x/I);
-	
-	if (is_exactly_a<function>(x)) {
-		const ex &t = x.op(0);
-
-		// tanh(atanh(x)) -> x
-		if (is_ex_the_function(x, atanh))
-			return t;
-
-		// tanh(asinh(x)) -> x/sqrt(1+x^2)
-		if (is_ex_the_function(x, asinh))
-			return t*power(_ex1+power(t,_ex2),_ex_1_2);
-
-		// tanh(acosh(x)) -> sqrt(x-1)*sqrt(x+1)/x
-		if (is_ex_the_function(x, acosh))
-			return sqrt(t-_ex1)*sqrt(t+_ex1)*power(t,_ex_1);
+	if (m_.is_integer_one()) {
+		return -log(1-x_);
 	}
-	
-	return tanh(x).hold();
+	if (m_.is_equal(_ex2)) {
+		if (x_.is_equal(I)) {
+			return power(Pi,_ex2)/_ex_48 + Catalan*I;
+		}
+		if (x_.is_equal(-I)) {
+			return power(Pi,_ex2)/_ex_48 - Catalan*I;
+		}
+	}
+
+        if ((is_exactly_a<numeric>(x_) and not ex_to<numeric>(x_).is_exact())
+            or (is_exactly_a<numeric>(m_) and not ex_to<numeric>(m_).is_exact())) {
+	        return Li_evalf(m_, x_, nullptr);
+	}
+
+	return Li(m_, x_).hold();
 }
 
-static ex tanh_deriv(const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param==0);
-	
-	// d/dx tanh(x) -> 1-tanh(x)^2
-	return _ex1-power(tanh(x),_ex2);
-}
 
-static ex tanh_series(const ex &x,
-                      const relational &rel,
-                      int order,
-                      unsigned options)
+static ex Li_series(const ex& m, const ex& x, const relational& rel, int order, unsigned options)
 {
-	GINAC_ASSERT(is_a<symbol>(rel.lhs()));
-	// method:
-	// Taylor series where there is no pole falls back to tanh_deriv.
-	// On a pole simply expand sinh(x)/cosh(x).
+	// classical polylog
 	const ex x_pt = x.subs(rel, subs_options::no_pattern);
-	if (!(2*I*x_pt/Pi).info(info_flags::odd))
-		throw do_taylor();  // caught by function::series()
-	// if we got here we have to care for a simple pole
-	return (sinh(x)/cosh(x)).series(rel, order, options);
-}
-
-static ex tanh_real_part(const ex & x)
-{
-	ex a = GiNaC::real_part(x);
-	ex b = GiNaC::imag_part(x);
-	return tanh(a)/(1+power(tanh(a),2)*power(tan(b),2));
-}
-
-static ex tanh_imag_part(const ex & x)
-{
-	ex a = GiNaC::real_part(x);
-	ex b = GiNaC::imag_part(x);
-	return tan(b)/(1+power(tanh(a),2)*power(tan(b),2));
-}
-
-static ex tanh_conjugate(const ex & x)
-{
-	// conjugate(tanh(x))==tanh(conjugate(x))
-	return tanh(x.conjugate());
-}
-
-REGISTER_FUNCTION(tanh, eval_func(tanh_eval).
-                        evalf_func(tanh_evalf).
-                        derivative_func(tanh_deriv).
-                        series_func(tanh_series).
-                        real_part_func(tanh_real_part).
-                        imag_part_func(tanh_imag_part).
-                        conjugate_func(tanh_conjugate).
-                        latex_name("\\tanh"));
-
-//////////
-// inverse hyperbolic sine (trigonometric function)
-//////////
-
-static ex asinh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return asinh(ex_to<numeric>(x));
-	
-	return asinh(x).hold();
-}
-
-static ex asinh_eval(const ex & x)
-{
-	if (x.info(info_flags::numeric)) {
-
-		// asinh(0) -> 0
-		if (x.is_zero())
-			return _ex0;
-
-		// asinh(float) -> float
-		if (!x.info(info_flags::crational))
-			return asinh(ex_to<numeric>(x));
-
-		// asinh() is odd
-		if (x.info(info_flags::negative))
-			return -asinh(-x);
+	if (m.info(info_flags::numeric) && x_pt.info(info_flags::numeric)) {
+		// First special case: x==0 (derivatives have poles)
+		if (x_pt.is_zero()) {
+			const symbol s;
+			ex ser;
+			// manually construct the primitive expansion
+			for (int i=1; i<order; ++i)
+				ser += pow(s,i) / pow(numeric(i), m);
+			// substitute the argument's series expansion
+			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
+			// maybe that was terminating, so add a proper order term
+			epvector nseq;
+			nseq.push_back(expair(Order(_ex1), order));
+			ser += pseries(rel, nseq);
+			// reexpanding it will collapse the series again
+			return ser.series(rel, order);
+		}
+		// TODO special cases: x==1 (branch point) and x real, >=1 (branch cut)
+		throw std::runtime_error("Li_series: don't know how to do the series expansion at this point!");
 	}
-	
-	// asinh(oo) -> oo
-	// asinh(-oo) -> -oo
-	// asinh(UnsignedInfinity) -> UnsignedInfinity
-	if (x.info(info_flags::infinity)) {
-		return x;
+	// all other cases should be safe, by now:
+	throw do_taylor();  // caught by function::series()
+}
+
+
+static ex Li_deriv(const ex& m_, const ex& x_, unsigned deriv_param)
+{
+	GINAC_ASSERT(deriv_param < 2);
+	if (deriv_param == 0) {
+		return _ex0;
 	}
-
-	return asinh(x).hold();
-}
-
-static ex asinh_deriv(const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param==0);
-	
-	// d/dx asinh(x) -> 1/sqrt(1+x^2)
-	return power(_ex1+power(x,_ex2),_ex_1_2);
-}
-
-static ex asinh_conjugate(const ex & x)
-{
-	// conjugate(asinh(x))==asinh(conjugate(x)) unless on the branch cuts which
-	// run along the imaginary axis outside the interval [-I, +I].
-	if (x.info(info_flags::real))
-		return asinh(x);
-	if (is_exactly_a<numeric>(x)) {
-		const numeric x_re = ex_to<numeric>(x.real_part());
-		const numeric x_im = ex_to<numeric>(x.imag_part());
-		if (!x_re.is_zero() ||
-		    (x_im > *_num_1_p && x_im < *_num1_p))
-			return asinh(x.conjugate());
+	ex m = m_;
+	ex x = x_;
+	if (m > 0) {
+		return Li(m-1, x) / x;
+	} else {
+		return 1/(1-x);
 	}
-	return conjugate_function(asinh(x)).hold();
 }
 
-REGISTER_FUNCTION(asinh, eval_func(asinh_eval).
-                         evalf_func(asinh_evalf).
-                         derivative_func(asinh_deriv).
-                         conjugate_func(asinh_conjugate).
-			 set_name("arcsinh"));
 
-//////////
-// inverse hyperbolic cosine (trigonometric function)
-//////////
-
-static ex acosh_evalf(const ex & x, PyObject* parent)
+static void Li_print_latex(const ex& m_, const ex& x_, const print_context& c)
 {
-	if (is_exactly_a<numeric>(x))
-		return acosh(ex_to<numeric>(x));
-	
-	return acosh(x).hold();
-}
-
-static ex acosh_eval(const ex & x)
-{
-	if (x.info(info_flags::numeric)) {
-
-		// acosh(0) -> Pi*I/2
-		if (x.is_zero())
-			return Pi*I*numeric(1,2);
-
-		// acosh(1) -> 0
-		if (x.is_equal(_ex1))
-			return _ex0;
-
-		// acosh(-1) -> Pi*I
-		if (x.is_equal(_ex_1))
-			return Pi*I;
-
-		// acosh(float) -> float
-		if (!x.info(info_flags::crational))
-			return acosh(ex_to<numeric>(x));
-
-		// acosh(-x) -> Pi*I-acosh(x)
-		if (x.info(info_flags::negative))
-			return Pi*I-acosh(-x);
+	lst m;
+	if (is_a<lst>(m_)) {
+		m = ex_to<lst>(m_);
+	} else {
+		m = lst(m_);
 	}
-	
-	// acosh(oo) -> oo
-	// acosh(-oo) -> oo
-	// acosh(UnsignedInfinity) -> oo
-	if (x.info(info_flags::infinity)) {
-		return Infinity;
+	lst x;
+	if (is_a<lst>(x_)) {
+		x = ex_to<lst>(x_);
+	} else {
+		x = lst(x_);
 	}
-
-	return acosh(x).hold();
-}
-
-static ex acosh_deriv(const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param==0);
-	
-	// d/dx acosh(x) -> 1/(sqrt(x-1)*sqrt(x+1))
-	return power(x+_ex_1,_ex_1_2)*power(x+_ex1,_ex_1_2);
-}
-
-static ex acosh_conjugate(const ex & x)
-{
-	// conjugate(acosh(x))==acosh(conjugate(x)) unless on the branch cut
-	// which runs along the real axis from +1 to -inf.
-	if (is_exactly_a<numeric>(x) &&
-	    (!x.imag_part().is_zero() || x > *_num1_p)) {
-		return acosh(x.conjugate());
+	c.s << "{\\rm Li}_{";
+	auto itm = m.begin();
+	(*itm).print(c);
+	itm++;
+	for (; itm != m.end(); itm++) {
+		c.s << ",";
+		(*itm).print(c);
 	}
-	return conjugate_function(acosh(x)).hold();
-}
-
-REGISTER_FUNCTION(acosh, eval_func(acosh_eval).
-                         evalf_func(acosh_evalf).
-                         derivative_func(acosh_deriv).
-                         conjugate_func(acosh_conjugate).
-			 set_name("arccosh"));
-
-//////////
-// inverse hyperbolic tangent (trigonometric function)
-//////////
-
-static ex atanh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return atanh(ex_to<numeric>(x));
-	
-	return atanh(x).hold();
-}
-
-static ex atanh_eval(const ex & x)
-{
-	if (x.info(info_flags::numeric)) {
-
-		// atanh(0) -> 0
-		if (x.is_zero())
-			return _ex0;
-
-		/*
-		// atanh({+|-}1) -> throw
-		if (x.is_equal(_ex1) || x.is_equal(_ex_1))
-			throw (pole_error("atanh_eval(): logarithmic pole",0));
-		*/
-
-		// atanh(1) -> oo
-		if (x.is_equal(_ex1))
-			return Infinity;
-		// atahn(-1) -> -oo
-		if (x.is_equal(_ex_1))
-			return NegInfinity;
-
-		// atanh(float) -> float
-		if (!x.info(info_flags::crational))
-			return atanh(ex_to<numeric>(x));
-
-		// atanh() is odd
-		if (x.info(info_flags::negative))
-			return -atanh(-x);
+	c.s << "}(";
+	auto itx = x.begin();
+	(*itx).print(c);
+	itx++;
+	for (; itx != x.end(); itx++) {
+		c.s << ",";
+		(*itx).print(c);
 	}
-	
-	// atanh(oo) -> -i*pi/2
-	// atanh(-oo) -> i*pi/2
-	// atanh(UnsignedInfinity) -> error
-	if (x.info(info_flags::infinity)) {
-		if (x.is_equal(Infinity))
-			return _ex_1_2*Pi*I;
-		if (x.is_equal(NegInfinity))
-			return _ex1_2*Pi*I;
-		// x is UnsignedInfinity
-		throw (std::runtime_error("arctanh_eval(): arctanh(unsigned_infinity) encountered"));
-	}
-		
-	return atanh(x).hold();
+	c.s << ")";
 }
 
-static ex atanh_deriv(const ex & x, unsigned deriv_param)
-{
-	GINAC_ASSERT(deriv_param==0);
-	
-	// d/dx atanh(x) -> 1/(1-x^2)
-	return power(_ex1-power(x,_ex2),_ex_1);
-}
-
-static ex atanh_series(const ex &arg,
-                       const relational &rel,
-                       int order,
-                       unsigned options)
-{
-	GINAC_ASSERT(is_a<symbol>(rel.lhs()));
-	// method:
-	// Taylor series where there is no pole or cut falls back to atanh_deriv.
-	// There are two branch cuts, one runnig from 1 up the real axis and one
-	// one running from -1 down the real axis.  The points 1 and -1 are poles
-	// On the branch cuts and the poles series expand
-	//     (log(1+x)-log(1-x))/2
-	// instead.
-	const ex arg_pt = arg.subs(rel, subs_options::no_pattern);
-	if (!(arg_pt).info(info_flags::real))
-		throw do_taylor();     // Im(x) != 0
-	if ((arg_pt).info(info_flags::real) && abs(arg_pt)<_ex1)
-		throw do_taylor();     // Im(x) == 0, but abs(x)<1
-	// care for the poles, using the defining formula for atanh()...
-	if (arg_pt.is_equal(_ex1) || arg_pt.is_equal(_ex_1))
-		return ((log(_ex1+arg)-log(_ex1-arg))*_ex1_2).series(rel, order, options);
-	// ...and the branch cuts (the discontinuity at the cut being just I*Pi)
-	if (!(options & series_options::suppress_branchcut)) {
- 		// method:
- 		// This is the branch cut: assemble the primitive series manually and
- 		// then add the corresponding complex step function.
- 		const symbol &s = ex_to<symbol>(rel.lhs());
- 		const ex &point = rel.rhs();
- 		const symbol foo;
- 		const ex replarg = series(atanh(arg), s==foo, order).subs(foo==point, subs_options::no_pattern);
-		ex Order0correction = replarg.op(0)+csgn(I*arg)*Pi*I*_ex1_2;
-		if (arg_pt<_ex0)
-			Order0correction += log((arg_pt+_ex_1)/(arg_pt+_ex1))*_ex1_2;
-		else
-			Order0correction += log((arg_pt+_ex1)/(arg_pt+_ex_1))*_ex_1_2;
- 		epvector seq;
-		seq.push_back(expair(Order0correction, _ex0));
- 		seq.push_back(expair(Order(_ex1), order));
- 		return series(replarg - pseries(rel, seq), rel, order);
-	}
-	throw do_taylor();
-}
-
-static ex atanh_conjugate(const ex & x)
-{
-	// conjugate(atanh(x))==atanh(conjugate(x)) unless on the branch cuts which
-	// run along the real axis outside the interval [-1, +1].
-	if (is_exactly_a<numeric>(x) &&
-	    (!x.imag_part().is_zero() || (x > *_num_1_p && x < *_num1_p))) {
-		return atanh(x.conjugate());
-	}
-	return conjugate_function(atanh(x)).hold();
-}
-
-REGISTER_FUNCTION(atanh, eval_func(atanh_eval).
-                         evalf_func(atanh_evalf).
-                         derivative_func(atanh_deriv).
-                         series_func(atanh_series).
-                         conjugate_func(atanh_conjugate).
-			 set_name("arctanh"));
 
+unsigned Li_SERIAL::serial = function::register_new(function_options("polylog", 2).
+                  evalf_func(Li_evalf).
+                  eval_func(Li_eval).
+                  series_func(Li_series).
+                  derivative_func(Li_deriv).
+                  print_func<print_latex>(Li_print_latex).
+                  do_not_evalf_params());
 
 } // namespace GiNaC
