@@ -23,7 +23,6 @@
 #include "mul.h"
 #include "add.h"
 #include "operators.h"
-#include "matrix.h"
 #include "indexed.h"
 #include "lst.h"
 #include "archive.h"
@@ -364,7 +363,7 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 			print_latex tcontext(tstream, c.options);
 			print_exvector(others, tcontext, sep);
 			print_overall_coeff(overall_coeff, c,
-					std::isdigit(tstream.peek()) ?
+					std::isdigit(tstream.peek()) != 0 ?
 						" \\cdot " : " \\, ",
 					latex_tags);
 			c.s<<tstream.str();
@@ -640,13 +639,6 @@ ex mul::coeff(const ex & s, int n) const
  *  @param level cut-off in recursive evaluation */
 ex mul::eval(int level) const
 {
-	std::unique_ptr<epvector> evaled_seqp = evalchildren(level);
-	if (evaled_seqp.get()) {
-		// do more evaluation later
-		return (new mul(*evaled_seqp, overall_coeff))->
-		           setflag(status_flags::dynallocated);
-	}
-	
 #ifdef DO_GINAC_ASSERT
         for (const auto & elem : seq) {
 		GINAC_ASSERT((!is_exactly_a<mul>(elem.rest)) ||
@@ -665,12 +657,19 @@ ex mul::eval(int level) const
 	}
 #endif // def DO_GINAC_ASSERT
 	
-	if (flags & status_flags::evaluated) {
+	if ((level == 1) and (flags & status_flags::evaluated) != 0u) {
 		GINAC_ASSERT(seq.size()>0);
 		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_integer_one());
 		return *this;
 	}
 
+	std::unique_ptr<epvector> evaled_seqp = evalchildren(level);
+	if (unlikely(evaled_seqp != nullptr)) {
+		// do more evaluation later
+		return (new mul(*evaled_seqp, overall_coeff))->
+		           setflag(status_flags::dynallocated);
+	}
+	
 	// handle infinity and handle exp(a)*exp(b) -> exp(a+b) and
 	unsigned exp_count = 0;
 	for (auto i = seq.begin(); i != seq.end(); i++) {
@@ -718,7 +717,7 @@ ex mul::eval(int level) const
                         return result.overall_coeff;
                 else
                         return x;
-	} else if ((seq_size >= 2) && (! (flags & status_flags::expanded))) {
+	} else if ((seq_size >= 2) && ((flags & status_flags::expanded) == 0u)) {
 		// Strip the content and the unit part from each term. Thus
 		// things like (-x+a)*(3*x-3*a) automagically turn into - 3*(x-a)2
 
@@ -839,8 +838,9 @@ ex mul::eval_infinity(epvector::const_iterator infinity_iter) const
 {
 	GINAC_ASSERT(is_exactly_a<infinity>(infinity_iter->rest));
 	GINAC_ASSERT(infinity_iter->coeff.is_equal(_ex1));
-	infinity result = ex_to<infinity>(recombine_pair_to_ex(*infinity_iter));
-	result *= overall_coeff;
+	infinity result(ex_to<numeric>(
+                ex_to<infinity>(infinity_iter->rest).get_direction()));
+        result *= overall_coeff;
 
 	for (auto i = seq.begin(); i != seq.end(); i++) {
 		if (i == infinity_iter) continue;
@@ -863,9 +863,9 @@ ex mul::evalf(int level, PyObject* parent) const
 
 	--level;
         for (const auto & elem : seq)
-		s.push_back(combine_ex_with_coeff_to_pair(
-                elem.rest.evalf(level, parent), elem.coeff));
-	return mul(s, overall_coeff.evalf(level, parent));
+               s.push_back(combine_ex_with_coeff_to_pair(
+                        elem.rest.evalf(level, parent), elem.coeff));
+        return mul(s, overall_coeff.evalf(level, parent));
 }
 
 void mul::find_real_imag(ex & rp, ex & ip) const
@@ -903,44 +903,6 @@ ex mul::imag_part() const
 	return ip;
 }
 
-ex mul::evalm() const
-{
-	// numeric*matrix
-	if (seq.size() == 1 && seq[0].coeff.is_equal(_ex1)
-	 && is_exactly_a<matrix>(seq[0].rest))
-		return ex_to<matrix>(seq[0].rest).mul(ex_to<numeric>(overall_coeff));
-
-	// Evaluate children first, look whether there are any matrices at all
-	// (there can be either no matrices or one matrix; if there were more
-	// than one matrix, it would be a non-commutative product)
-	epvector s;
-	s.reserve(seq.size());
-
-	bool have_matrix = false;
-	epvector::iterator the_matrix;
-
-        for (const auto & elem : seq) {
-		const ex &m = recombine_pair_to_ex(elem).evalm();
-		s.push_back(split_ex_to_pair(m));
-		if (is_exactly_a<matrix>(m)) {
-			have_matrix = true;
-			the_matrix = s.end() - 1;
-		}
-	}
-
-	if (have_matrix) {
-
-		// The product contained a matrix. We will multiply all other factors
-		// into that matrix.
-		matrix m = ex_to<matrix>(the_matrix->rest);
-		s.erase(the_matrix);
-		ex scalar = (new mul(s, overall_coeff))->setflag(status_flags::dynallocated);
-		return m.mul_scalar(scalar);
-
-	} else
-		return (new mul(s, overall_coeff))->setflag(status_flags::dynallocated);
-}
-
 ex mul::eval_ncmul(const exvector & v) const
 {
 	if (seq.empty())
@@ -1038,7 +1000,7 @@ bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, lst &repls,
 
 bool mul::has(const ex & pattern, unsigned options) const
 {
-	if(!(options&has_options::algebraic))
+	if((options&has_options::algebraic) == 0u)
 		return basic::has(pattern,options);
 	if(is_exactly_a<mul>(pattern)) {
 		lst repls;
@@ -1171,7 +1133,7 @@ ex mul::derivative(const symbol & s) const
  Beware that symbolic exponents are wrapped inside pow objects with 1 as coeff. */
 double mul::total_degree() const
 {
-	if (flags & status_flags::tdegree_calculated) {
+	if ((flags & status_flags::tdegree_calculated) != 0u) {
 		return tdegree;
 	}
 	numeric tdeg = calc_total_degree();
@@ -1259,7 +1221,13 @@ expair mul::split_ex_to_pair(const ex & e) const
 expair mul::combine_ex_with_coeff_to_pair(const ex & e,
                                           const ex & c) const
 {
-	// to avoid duplication of power simplification rules,
+        GINAC_ASSERT(is_exactly_a<numeric>(c));
+
+        // First, try a common shortcut:
+        if (is_exactly_a<symbol>(e))
+                return expair(e, c);
+
+        // to avoid duplication of power simplification rules,
 	// we create a temporary power object
 	// otherwise it would be hard to correctly evaluate
 	// expression like (4^(1/3))^(3/2)
@@ -1272,18 +1240,27 @@ expair mul::combine_ex_with_coeff_to_pair(const ex & e,
 expair mul::combine_pair_with_coeff_to_pair(const expair & p,
                                             const ex & c) const
 {
+        GINAC_ASSERT(is_exactly_a<numeric>(p.coeff));
+        GINAC_ASSERT(is_exactly_a<numeric>(c));
+
+        if (is_exactly_a<symbol>(p.rest))
+                return expair(p.rest, p.coeff*c);
+	if (c.is_integer_one())
+		return p;
+        if (p.coeff.is_integer_one())
+                return expair(p.rest, c);
+
 	// to avoid duplication of power simplification rules,
 	// we create a temporary power object
 	// otherwise it would be hard to correctly evaluate
 	// expression like (4^(1/3))^(3/2)
-	if (c.is_integer_one())
-		return p;
-
 	return split_ex_to_pair(power(recombine_pair_to_ex(p),c));
 }
 	
 ex mul::recombine_pair_to_ex(const expair & p) const
 {
+        if (unlikely(is_exactly_a<infinity>(p.rest)))
+                return (new infinity(ex_to<numeric>(p.coeff)))->setflag(status_flags::evaluated|status_flags::dynallocated);
 	if (p.coeff.is_integer_one()) 
 		return p.rest;
 	else
@@ -1336,10 +1313,9 @@ void mul::combine_overall_coeff(const ex & c1, const ex & c2)
 bool mul::can_make_flat(const expair & p) const
 {
 	GINAC_ASSERT(is_exactly_a<numeric>(p.coeff));
-	// this assertion will probably fail somewhere
-	// it would require a more careful make_flat, obeying the power laws
-	// probably should return true only if p.coeff is integer
-	return p.coeff.is_integer_one();
+
+        // (x*y)^c == x^c*y^c  if c ∈ ℤ
+        return p.coeff.info(info_flags::integer);
 }
 
 bool mul::can_be_further_expanded(const ex & e)
@@ -1372,15 +1348,15 @@ ex mul::expand(unsigned options) const
         }
 
 	// do not rename indices if the object has no indices at all
-	if ((!(options & expand_options::expand_rename_idx)) && 
+	if (((options & expand_options::expand_rename_idx) == 0u) && 
 			this->info(info_flags::has_indices))
 		options |= expand_options::expand_rename_idx;
 
-	const bool skip_idx_rename = !(options & expand_options::expand_rename_idx);
+	const bool skip_idx_rename = (options & expand_options::expand_rename_idx) == 0u;
 
 	// First, expand the children
 	std::unique_ptr<epvector> expanded_seqp = expandchildren(options);
-	const epvector & expanded_seq = (expanded_seqp.get() ? *expanded_seqp : seq);
+	const epvector & expanded_seq = (expanded_seqp.get() != nullptr ? *expanded_seqp : seq);
 
 	// Now, look for all the factors that are sums and multiply each one out
 	// with the next one that is found while collecting the factors which are
@@ -1595,4 +1571,23 @@ std::unique_ptr<epvector> mul::expandchildren(unsigned options) const
 	return std::unique_ptr<epvector>(nullptr); // nothing has changed
 }
 
+ex mul::without_known_factor(const ex& f) const
+{
+	epvector s;
+	s.reserve(seq.size()-1);
+
+        bool found = false;
+	for (const auto & elem : seq) {
+                ex t = recombine_pair_to_ex(elem);
+                if (not found and t.is_equal(f)) {
+                        found = true;
+                }
+                else
+			s.push_back(elem);
+	}
+
+	mul * result = new mul(s, ex_to<numeric>(overall_coeff));
+	return result->setflag(status_flags::dynallocated);
+}
+
 } // namespace GiNaC
