@@ -33,6 +33,7 @@
 #include "integral.h"
 #include "archive.h"
 #include "utils.h"
+#include "infinity.h"
 
 #include <numeric>
 #include <stdexcept>
@@ -155,7 +156,7 @@ void pseries::print_series(const print_context & c, const char *openbrace, const
 					c.s << ')' << closebrace;
 				} else
 					var.print(c);
-				if (i->coeff.compare(_ex1)) {
+				if (i->coeff.compare(_ex1) != 0) {
 					c.s << pow_sym;
 					c.s << openbrace;
 					if (i->coeff.info(info_flags::negative)) {
@@ -215,7 +216,7 @@ void pseries::do_print_python_repr(const print_python_repr & c, unsigned level)
 	c.s << "),[";
 	size_t num = seq.size();
 	for (size_t i=0; i<num; ++i) {
-		if (i)
+		if (i != 0u)
 			c.s << ',';
 		c.s << '(';
 		seq[i].rest.print(c);
@@ -239,17 +240,17 @@ int pseries::compare_same_type(const basic & other) const
 	
 	// ...then the expansion point...
 	int cmpval = var.compare(o.var);
-	if (cmpval)
+	if (cmpval != 0)
 		return cmpval;
 	cmpval = point.compare(o.point);
-	if (cmpval)
+	if (cmpval != 0)
 		return cmpval;
 	
 	// ...and if that failed the individual elements
 	auto it = seq.begin(), o_it = o.seq.begin();
 	while (it!=seq.end() && o_it!=o.seq.end()) {
 		cmpval = it->compare(*o_it);
-		if (cmpval)
+		if (cmpval != 0)
 			return cmpval;
 		++it;
 		++o_it;
@@ -266,7 +267,7 @@ size_t pseries::nops() const
 }
 
 /** Return the ith term in the series when represented as a sum. */
-ex pseries::op(size_t i) const
+const ex pseries::op(size_t i) const
 {
 	if (i >= seq.size())
 		throw (std::out_of_range("op() out of range"));
@@ -283,7 +284,7 @@ int pseries::degree(const ex &s) const
 {
 	if (var.is_equal(s)) {
 		// Return last exponent
-		if (seq.size())
+		if (seq.size() != 0u)
 			return ex_to<numeric>((seq.end()-1)->coeff).to_int();
 		else
 			return 0;
@@ -310,7 +311,7 @@ int pseries::ldegree(const ex &s) const
 {
 	if (var.is_equal(s)) {
 		// Return first exponent
-		if (seq.size())
+		if (seq.size() != 0u)
 			return ex_to<numeric>((seq.begin())->coeff).to_int();
 		else
 			return 0;
@@ -456,7 +457,7 @@ ex pseries::eval_integ() const
 {
 	epvector *newseq = nullptr;
 	for (auto i=seq.begin(); i!=seq.end(); ++i) {
-		if (newseq) {
+		if (newseq != nullptr) {
 			newseq->push_back(expair(i->rest.eval_integ(), i->coeff));
 			continue;
 		}
@@ -471,40 +472,12 @@ ex pseries::eval_integ() const
 	}
 
 	ex newpoint = point.eval_integ();
-	if (newseq || !are_ex_trivially_equal(newpoint, point))
+	if ((newseq != nullptr) || !are_ex_trivially_equal(newpoint, point))
 		return (new pseries(var==newpoint, *newseq))
 		       ->setflag(status_flags::dynallocated);
 	return *this;
 }
 
-ex pseries::evalm() const
-{
-	// evalm each coefficient
-	epvector newseq;
-	bool something_changed = false;
-	for (auto i=seq.begin(); i!=seq.end(); ++i) {
-		if (something_changed) {
-			ex newcoeff = i->rest.evalm();
-			if (!newcoeff.is_zero())
-				newseq.push_back(expair(newcoeff, i->coeff));
-		}
-		else {
-			ex newcoeff = i->rest.evalm();
-			if (!are_ex_trivially_equal(newcoeff, i->rest)) {
-				something_changed = true;
-				newseq.reserve(seq.size());
-				std::copy(seq.begin(), i, std::back_inserter<epvector>(newseq));
-				if (!newcoeff.is_zero())
-					newseq.push_back(expair(newcoeff, i->coeff));
-			}
-		}
-	}
-	if (something_changed)
-		return (new pseries(var==point, newseq))->setflag(status_flags::dynallocated);
-	else
-		return *this;
-}
-
 ex pseries::subs(const exmap & m, unsigned options) const
 {
 	// If expansion variable is being substituted, convert the series to a
@@ -636,7 +609,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 		// We need to test for zero in order to see if the series terminates.
 		// The problem is that there is no such thing as a perfect test for
 		// zero.  Expanding the term occasionally helps a little...
-		deriv = deriv.diff(s).expand();
+		deriv = deriv.diff(s).expand(expand_options::expand_only_numerators);
 		if (deriv.is_zero())  // Series terminates
 			return pseries(r, seq);
 
@@ -647,7 +620,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 	
 	// Higher-order terms, if present
 	deriv = deriv.diff(s);
-	if (!deriv.expand().is_zero())
+        if (!deriv.expand(expand_options::expand_only_numerators).is_zero())
 		seq.push_back(expair(Order(_ex1), n));
 	return pseries(r, seq);
 }
@@ -1084,20 +1057,27 @@ pseries pseries::shift_exponents(int deg) const
 ex power::series(const relational & r, int order, unsigned options) const
 {
 	// If basis is already a series, just power it
-	if (is_exactly_a<pseries>(basis))
+	if (is_exactly_a<pseries>(basis)
+            and is_exactly_a<numeric>(exponent))
 		return ex_to<pseries>(basis).power_const(ex_to<numeric>(exponent), order);
 
 	// Basis is not a series, may there be a singularity?
 	bool must_expand_basis = false;
 	try {
-		basis.subs(r, subs_options::no_pattern);
+		ex basis_subs = basis.subs(r, subs_options::no_pattern);
+		if (is_exactly_a<infinity>(basis_subs)) {
+			must_expand_basis = true;
+		}
 	} catch (pole_error) {
 		must_expand_basis = true;
 	}
 
 	bool exponent_is_regular = true;
 	try {
-		exponent.subs(r, subs_options::no_pattern);
+		ex exponent_subs = exponent.subs(r, subs_options::no_pattern);
+		if (is_exactly_a<infinity>(exponent_subs)) {
+			exponent_is_regular = false;
+		}
 	} catch (pole_error) {
 		exponent_is_regular = false;
 	}
@@ -1131,7 +1111,8 @@ ex power::series(const relational & r, int order, unsigned options) const
 	// Singularity encountered, is the basis equal to (var - point)?
 	if (basis.is_equal(r.lhs() - r.rhs())) {
 		epvector new_seq;
-		if (ex_to<numeric>(exponent).to_int() < order)
+		if (is_exactly_a<numeric>(exponent)
+                    and ex_to<numeric>(exponent).to_int() < order)
 			new_seq.push_back(expair(_ex1, exponent));
 		else
 			new_seq.push_back(expair(Order(_ex1), exponent));
