@@ -183,7 +183,7 @@ bool relational::info(unsigned inf) const
 {
 	switch (inf) {
 		case info_flags::relation:
-			return 1;
+			return true;
 		case info_flags::relation_equal:
 			return o==equal;
 		case info_flags::relation_not_equal:
@@ -197,7 +197,7 @@ bool relational::info(unsigned inf) const
 		case info_flags::relation_greater_or_equal:
 			return o==greater_or_equal;
 	}
-	return 0;
+	return false;
 }
 
 size_t relational::nops() const
@@ -205,7 +205,14 @@ size_t relational::nops() const
 	return 2;
 }
 
-ex relational::op(size_t i) const
+const ex relational::op(size_t i) const
+{
+	GINAC_ASSERT(i<2);
+
+	return i==0 ? lh : rh;
+}
+
+ex & relational::let_op(size_t i)
 {
 	GINAC_ASSERT(i<2);
 
@@ -337,7 +344,7 @@ long relational::calchash() const
 	v ^= lhash;
 
 	// store calculated hash value only if object is already evaluated
-	if (flags & status_flags::evaluated) {
+	if ((flags & status_flags::evaluated) != 0u) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
 	}
@@ -471,25 +478,25 @@ relational::result relational::decide() const
                         else
                                 return result::notimplemented;
                 case less:
-                        if (df.info(info_flags::nonzero) and (-df).info(info_flags::positive))
-                                return result::True;
-                        else
-                                return result::notimplemented;
-                case greater:
-                        if (df.info(info_flags::nonzero) and df.info(info_flags::positive))
-                                return result::True;
-                        else
-                                return result::notimplemented;
-		case less_or_equal:
                         if ((-df).info(info_flags::positive))
                                 return result::True;
                         else
                                 return result::notimplemented;
-		case greater_or_equal:
+                case greater:
                         if (df.info(info_flags::positive))
                                 return result::True;
                         else
                                 return result::notimplemented;
+		case less_or_equal:
+                        if (df.is_zero() or (-df).info(info_flags::positive))
+                                return result::True;
+                        else
+                                return result::notimplemented;
+		case greater_or_equal:
+                        if (df.is_zero() or df.info(info_flags::positive))
+                                return result::True;
+                        else
+                                return result::notimplemented;
 		default:
                         throw(std::logic_error("invalid relational operator"));
                 }
